#! /usr/bin/perl -w

#=============================================================================
#    G I M S  A D M I N  .  C G I
#
#    GIMS project administration tool
#    Written by C. Thomas
#
#    $Id: GIMSAdmin.cgi,v 1.7 2010/07/14 22:11:30 cthomas Exp $
#=============================================================================


#=============================================================================
#    U S E  /  R E Q U I R E
#=============================================================================
use CGI qw(:standard);
use CGI::Carp ('fatalsToBrowser');         # Send any errors to the browser
use XML::RPC;
use XML::Simple qw(:strict);               # for converting XML <=> hash
use Data::Dumper;                          # debugging data structures
use POSIX;                                 # for strftime
use CGI::Cookie;                           # to handle cookies
use XmlRpcUtils;                           # for writing XML/RPC code
use GIMSDBUtils;                           # to connect to database
use GIMSJavascript;                        # for javascript magic
use strict;


#=============================================================================
#    P R O T O T Y P E S
#=============================================================================
sub getParams();
sub printModeSelectForm($);
sub printEditDeviceForm($$;$;$);
sub printDeleteDeviceForm($$$);
sub processEditDevice($);
sub writeToLog(;$);
sub getTimestamp();
sub printJavascript();



#=============================================================================
#    C O N F I G
#=============================================================================
my $this_script = 'http://www.schooner.wail.wisc.edu/gims/GIMSAdmin.cgi';
my $agg_man_url = 'http://www.schooner.wail.wisc.edu/gims/gims_aggregate_manager.cgi';
my $log_file = 'gims_admin.log';
my $javascript_file = 'GIMSControl.js';


#=============================================================================
#    P A R A M E T E R S
#=============================================================================
my $mode = '';
my $device_location = '';
my $device_name = '';
my $device_type = '';
my $hostname = '';
my $device_port = '';
my $descr = '';

my $selected_loc = '';
my $selected_dev = '';


#=============================================================================
#    G L O B A L  V A R I A B L E S
#=============================================================================
my $dbh = undef;
my $cgi_obj = undef;
my $username = undef;
my $LOG = undef;
my %device_info = ();
my %cur_devices = ();
my @aggregation_types = ('none','combine_pkt_flows','count_pkts');
my %device_caps = ('storage_types'=>{'local'=>0,
                                     's3'=>0,
                                     'ssh'=>0},
                   'anon_types' => {'none'=>0,'anonymize'=>0},
                   'filter_types' => {'src_addr'=>0,'dst_addr'=>0,
                                      'src_port'=>0,'dst_port'=>0,
                                      'protocol'=>0,'vlan_num'=>0},
                   'protocol_types' => {'tcp'=>0,
                                        'udp'=>0,
                                        'icmp'=>0},
                   'sampling_types' => {'none'=>0,
                                        'everyN'=>0,
                                        'uniformrandom'=>0},
                   'aggregation_types' => {'none'=>0,
                                           'combine_pkt_flows'=>0,
                                           'count_pkts'=>0},
                   'misc' => {'libpcap_strings'=>0,
                              'metadata'=>0},
                  );


#=============================================================================
#    M A I N 
#=============================================================================

$| = 1; # set to unbuffered display

#===== Get a CGI object
$cgi_obj = new CGI or die "Unable to create CGI object!\n";

#===== Get all parameters passed into the script
getParams();

#===== Get a database handle
$dbh = GIMSDBUtils::getDBHandle();
die "Could not connect to database!\n" unless ref($dbh);

#==== Open log file
unless(open($LOG, ">>$log_file"))
{
  warn "Couldn't open log file! : $!\n";
}

#===== Print HTML header stuff
print "Content-type: text/html\n\n";
print qq~<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"\n~;
print qq~"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">\n~;
print qq~<html xmlns="http://www.w3.org/1999/xhtml" lang="en-US" xml:lang="en-US">\n~;
print qq~<link href="CSS/Standard.css" rel="stylesheet" type="text/css">
\n~;
print "<head><title>GIMSAdmin.cgi</title></head><body>\n";
print "<body bgcolor=#83a2b8>\n";

#===== Banner
print qq~<img src='images/GIMSBanner.jpg' alt='GIMS Banner' class='banner'>\n~;

#===== User logged in?
if ($username)
{
  print "<p><div class=small_font>User $username logged in.</div><p>\n";
}
else
{
  print "<p><div class=small_font>User authentication not obtained.</div><p>\n";
}

#===== Configure experiment mode 
if ($mode and $mode eq 'add_device')
{
  #===== Set defaults for device capabilities
  $device_caps{aggregation_types}{none} = 1;
  $device_caps{anon_types}{none} = 1;
  $device_caps{filter_types}{protocol} = 1;
  $device_caps{filter_types}{vlan_num} = 1;
  $device_caps{misc}{metadata} = 1;
  $device_caps{misc}{libpcap_strings} = 1;
  $device_caps{sampling_types}{none} = 1;
  $device_caps{storage_types}{local} = 1;

  GIMSDBUtils::getDeviceInfo($dbh, \%cur_devices);
  printEditDeviceForm(\%cur_devices, \%device_caps);
}
elsif ($mode and $mode eq 'edit_device')
{
  my $xml_code = '';

  #===== Get info for current devices
  GIMSDBUtils::getDeviceInfo($dbh, \%cur_devices);

  #===== Get the XML/RPC representation of the selected device's 
  #      capabilities
  $xml_code = $cur_devices{$selected_loc}{$selected_dev}{'capabilities'};

  #====== Set the device capabilities hash to current settings
  XmlRpcUtils::getDeviceCapsHash($xml_code, \%device_caps);

  #===== Print the device form
  printEditDeviceForm(\%cur_devices, \%device_caps, 
                      $selected_loc, $selected_dev);
}
elsif ($mode and $mode eq 'delete_device')
{
  GIMSDBUtils::getDeviceInfo($dbh, \%cur_devices);
  printDeleteDeviceForm(\%cur_devices, $selected_loc, $selected_dev);
}
elsif ($mode and $mode eq 'process_add_device')
{
  processEditDevice(\%device_info);
}
elsif ($mode and $mode eq 'process_edit_device')
{
  my %edit_device = ();

  GIMSDBUtils::getDeviceInfo($dbh, \%cur_devices);

  $edit_device{'location'} = $selected_loc;
  $edit_device{'device_name'} = $selected_dev;
  $edit_device{'type'} = $cur_devices{$selected_loc}{$selected_dev}{'type'};
  $edit_device{'hostname'} = $cur_devices{$selected_loc}{$selected_dev}{'hostname'};
  $edit_device{'port'} = $cur_devices{$selected_loc}{$selected_dev}{'port'};
  $edit_device{'descr'} = $cur_devices{$selected_loc}{$selected_dev}{'descr'};
  $edit_device{'capabilities'} = $device_info{'capabilities'};

  GIMSDBUtils::getDeviceInfo($dbh, \%cur_devices);
  processEditDevice(\%edit_device);
}
elsif ($mode and $mode eq 'process_delete_device')
{
  GIMSDBUtils::getDeviceInfo($dbh, \%cur_devices);
  processDeleteDevice(\%cur_devices, $selected_loc, $selected_dev);
}
else
{
  GIMSDBUtils::getDeviceInfo($dbh, \%cur_devices);
  printModeSelectForm(\%cur_devices);
}


#==== Print HTML footer stuff
print "<p><p>\n";
print end_html, "\n";

#===== Print javascript
printJavascript();

#===== Disconnect from database
$dbh->disconnect;

exit;

######################### BEGIN SUBROUTINES ##################################



#=============================================================================
#    G E T  P A R A M S
#=============================================================================
sub getParams()
{
  if ($ENV{REMOTE_USER})
  {
    $username = $ENV{REMOTE_USER};
  }
 
  #===== Get basic info   
  if (param('mode')){            $mode = param(             'mode');}
  if (param('device_location')){ $device_location = param(  'device_location');}
  if (param('device_name')){     $device_name = param(      'device_name');}
  if (param('device_type')){     $device_type = param(      'device_type');}
  if (param('hostname')){        $hostname = param(         'hostname');}
  if (param('device_port')){     $device_port = param(      'device_port');}
  if (param('descr')){           $descr = param(            'descr');}

  #===== Get capabilities
  my @params = $cgi_obj->param;
  foreach my $param (@params)
  {
    my ($struct, $member) = split('::', $param);
    if ($struct and $member)
    {
      push(@{$device_info{'capabilities'}{$struct}},$member);
    }
  }

  #===== Store basic info
  $device_info{'location'} = $device_location;
  $device_info{'device_name'} = $device_name;
  $device_info{'type'} = $device_type;
  $device_info{'hostname'} = $hostname;
  $device_info{'port'} = $device_port;
  $device_info{'descr'} = $descr;

  #===== Selected device
  if (param('selected_loc')){$selected_loc = param('selected_loc');}
  if (param('selected_dev'))
  {  
    $selected_dev = param( 'selected_dev');
    if ($selected_dev =~ m/\:\:/)
    {
      ($selected_loc, $selected_dev) = split('::', $selected_dev); 
    }
  }

  return;
  
} # end of getParams()


#=============================================================================
#     P R I N T  M O D E  S E L E C T  F O R M
#
#     Arg1: Hash ref containing device info
#=============================================================================
sub printModeSelectForm($)
{
  my $device_ref = shift or
    die "Must supply hash ref to printModeSelectForm()!\n";

  print "<center><h1>GIMS Administration Tool</h1></center>\n";
  print "<p><p>\n";

  print qq~<form name='myForm' method="post" action="$this_script">\n~;  

  #====== Mode select
  print "<p>Action to perform:<br>\n";
  print "<select name='mode' onClick='handleMode();'>\n";
  print "<option value='' selected>Select an action...\n";
  print "<option value='add_device'>Add a New Device\n";
  print "<option value='edit_device'>Edit an Existing Device\n"; 
  print "<option value='delete_device'>Delete a Device\n";
  print "</select>\n";

  print "<p><p>\n";

  #====== Device select
  print "<p>Select a Device:<br>\n";
  print "<select name='selected_dev' disabled>\n";
  print "<option value='' selected>Choose a device...\n";
  foreach my $location (sort keys %{$device_ref})
  {
    foreach my $dev (sort keys %{$device_ref->{$location}})
    {
      print "<option value='$location\:\:$dev'>$location:$dev\n"; 
    } # for each device
  } # for each location
  print "</select>\n";

  print "<p><p>\n";

  #===== Buttons
  print "<input type='button' onClick='submitModeSelect();' value='Submit'>\n";

  print "<p><hr><p>\n";

  #===== Return to start link
  print qq~<a href="$this_script">Return to Start</a>\n~;

  print "</form>\n";
  
  return;
  
} # end of printModeSelectForm()


#=============================================================================
#     P R I N T  E D I T  D E V I C E  F O R M
#=============================================================================
sub printEditDeviceForm($$;$;$)
{
  my $dev_info_ref = shift or
    die "Must supply device info hash ref to printEditDeviceForm()!\n";
  my $caps_ref = shift or
    die "Must supply device caps hash ref to printEditDeviceForm()!\n";
  my $edit_loc = shift;
  my $edit_dev = shift;
  my $edit_type = '';
  my $edit_hostname = '';
  my $edit_port = '';
  my $edit_descr = '';

  print "<center><h1>GIMS Administration Tool</h1></center>\n";
  print "<p><p>\n";

  #===== Get info
  if ($edit_loc and $edit_dev)
  {
    $edit_type = $dev_info_ref->{$edit_loc}{$edit_dev}{'type'};
    $edit_hostname = $dev_info_ref->{$edit_loc}{$edit_dev}{'hostname'};
    $edit_port = $dev_info_ref->{$edit_loc}{$edit_dev}{'port'};
    $edit_descr = $dev_info_ref->{$edit_loc}{$edit_dev}{'descr'};
  }

  print qq~<form name='myForm' method="post" action="$this_script">\n~;  

  print "<table>\n";

  print "<tr>\n";
  print "<td><strong>Location: </strong></td>\n";
  print qq~<td><select name='device_location' onChange='handleLocation();'>\n~;
  foreach my $location (sort keys %{$dev_info_ref})
  {
    if ($edit_loc and $location eq $edit_loc)
    {
      print "<option value='$location' selected>$location\n";
    }
    else
    {
      print "<option value='$location'>$location\n";
    }
  } # for each location
  print "<option value=''>Add a new location...\n";
  print "</select></td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<td><strong>Device Name: </strong></td>\n";
  print qq~<td><input name='device_name' size=35 value="$edit_dev"></td>\n~;
  print "</tr>\n";

  print "<tr>\n";
  print "<td><strong>Device Type: </strong></td>\n";
  print qq~<td><select name='device_type'>\n~;
  print "<option value='PKT_CAP' selected>Packet Capture Device\n";
  print "</select></td>\n";
  print "</tr>\n";

  print "<tr>\n";
  print "<td><strong>Hostname: </strong></td>\n";
  print qq~<td><input name='hostname' size=35 value="$edit_hostname"></td>\n~;
  print "</tr>\n";

  print "<tr>\n";
  print "<td><strong>Port: </strong></td>\n";
  print qq~<td><input name='device_port' size=6 value="$edit_port"></td>\n~;
  print "</tr>\n";

  print "<tr>\n";
  print "<td><strong>Description: </strong></td>\n";
  print qq~<td><textarea name='descr' rows=4 cols=72>\n~;
  print "$edit_descr" if $edit_descr;
  print "</textarea></td>\n";
  print "</tr>\n";

  print "</table>\n";
  
  print "<p><hr><p>\n";

  #===== Capabilities
  print "<strong>Capabilities (check types supported by device):</strong><p>\n";
  foreach my $heading (sort keys %{$caps_ref})
  {
    print "<strong>$heading</strong><br>\n";
    foreach my $sub_head (keys %{$caps_ref->{$heading}})
    {
      my $name = "$heading\:\:$sub_head";
      if ($caps_ref->{$heading}{$sub_head})
      {
        print "&nbsp;&nbsp;<input type='checkbox' name='$name' ";
        print "value='true' checked> ";
        print "$sub_head<br>\n";
      } # checked
      else
      {
        print "&nbsp;&nbsp;<input type='checkbox' name='$name' value='true'> ";
        print "$sub_head<br>\n";
      } # unchecked
    }
  }

  #===== Buttons 
  print "<hr>\n";
  print "<input type='button' onclick='submitEditDevice();' value='Submit'>\n";

  print "<p><hr><p>\n";

  #===== Return to start link
  print qq~<a href="$this_script">Return to Start</a>\n~;

  #===== Hidden
  if ($mode eq 'add_device')
  {
    print "<input type='hidden' name='mode' value='process_add_device'>\n";
  }
  else
  {
    print "<input type='hidden' name='mode' value='process_edit_device'>\n";
  }
  print "<input type='hidden' name='selected_loc' value='$edit_loc'>\n";
  print "<input type='hidden' name='selected_dev' value='$edit_dev'>\n";

  print "</form>\n";
  
  return;

} # end of printEditDeviceForm()


#=============================================================================
#     P R I N T  D E L E T E  D E V I C E  F O R M
#=============================================================================
sub printDeleteDeviceForm($$$)
{
  my $info_ref = shift or
    die "Must supply hash ref to printDeleteDeviceForm()!\n";
  my $del_loc = shift or
    die "Must supply a location to printDeleteDeviceForm()!\n";
  my $del_dev = shift or
    die "Must supply a device_name to printDeleteDeviceForm()!\n";

  print "<center><h1>GIMS Administration Tool</h1></center>\n";
  print "<p><p>\n";

  print qq~<form name='myForm' method="post" action="$this_script">\n~;  

  #===== Get info
  my $type = $info_ref->{$del_loc}{$del_dev}{'type'};
  my $hostname = $info_ref->{$del_loc}{$del_dev}{'hostname'};
  my $port = $info_ref->{$del_loc}{$del_dev}{'port'};
  my $descr = $info_ref->{$del_loc}{$del_dev}{'descr'};

  #===== Print device info
  print "<strong>Deleting Device:</strong><br>\n";
  print "&nbsp;&nbsp;Location: $del_loc<br>\n";
  print "&nbsp;&nbsp;Device Name: $del_dev<br>\n";
  print "&nbsp;&nbsp;Type: $type<br>\n";
  print "&nbsp;&nbsp;Hostname: $hostname<br>\n";
  print "&nbsp;&nbsp;Port: $port<br>\n";
  print "&nbsp;&nbsp;Description: $descr<br>\n";
  print "<p>\n";


  print "<p><p>\n";

  #===== Button
  print "<input type='button' onClick='submitDeleteDevice();'";
  print " value='Delete Device'>\n";

  print "<p><hr><p>\n";

  #===== Hidden
  print "<input type='hidden' name='selected_loc' value='$del_loc'>";
  print "<input type='hidden' name='selected_dev' value='$del_dev'>";
  print "<input type='hidden' name='mode' value='process_delete_device'>";

  #===== Return to start link
  print qq~<a href="$this_script">Return to Start</a>\n~;

  print "</form>\n";
  
  return;

} # end of printDeleteDeviceForm()


#=============================================================================
#     P R O C E S S  E D I T  D E V I C E
#     Arg1: Hash ref holding device info
#=============================================================================
sub processEditDevice($)
{
  my $info_ref = shift or
    die "Must supply hash ref to processEditDevice()!\n";
  my $log_str = '';

  #===== Get info from hash
  my $location = $info_ref->{'location'};
  my $device_name = $info_ref->{'device_name'};
  my $type = $info_ref->{'type'};
  my $hostname = $info_ref->{'hostname'};
  my $port = $info_ref->{'port'};
  my $descr = $info_ref->{'descr'};
  my $caps = $info_ref->{'capabilities'};

  #===== Get the XML code from the hash
  my $xml_code = XmlRpcUtils::getDeviceCapsXML($caps);
  $info_ref->{'capabilities'} = $xml_code;
  $xml_code =~ s/>/&gt;/g;
  $xml_code =~ s/</&lt;/g;


  #===== Write the change to the database
  my $errors = GIMSDBUtils::updateDevice($dbh, $info_ref);
  if ($errors)
  {
    print "ERROR: $errors!<p>\n";
  }
  else
  {
    print "Device update successful!<p>\n";
  }

  #===== Print the info sent to DB
  print "Location: $location<br>\n";
  print "Device Name: $device_name<br>\n";
  print "Type: $type<br>\n";
  print "Hostname: $hostname<br>\n";
  print "Port: $port<br>\n";
  print "Descr: $descr<br>\n";
  print "Capabilities (as XML/RPC):<br>\n";
  print "<pre>$xml_code</pre>";
  print "<p><p><hr><p>\n";

  #===== Return to start link
  print qq~<a href="$this_script">Return to Start</a>\n~;

  #===== Write to the log
  if ($mode eq 'process_add_device')
  {
    $log_str = "Added device $location:$device_name to database.\n";
  }
  else
  {
    $log_str = "Edited device $location:$device_name in database.\n";
  }
  writeToLog($log_str);

  return;

} # end of processEditDevice()


#=============================================================================
#     P R O C E S S  D E L E T E  D E V I C E
#     Arg1: Hash ref holding device info
#=============================================================================
sub processDeleteDevice($$$)
{
  my $info_ref = shift or
    die "Must supply hash ref to processDeleteDevice()!\n";
  my $del_loc = shift or
    die "Must supply location to processDeleteDevice()!\n";
  my $del_dev = shift or
    die "Must supply device_name to processDeleteDevice()!\n";

  #===== Get info from hash
  my $type = $info_ref->{$del_loc}{$del_dev}{'type'};
  my $hostname = $info_ref->{$del_loc}{$del_dev}{'hostname'};
  my $port = $info_ref->{$del_loc}{$del_dev}{'port'};
  my $descr = $info_ref->{$del_loc}{$del_dev}{'descr'};

  #===== Update the database
  my $errors = GIMSDBUtils::deleteDevice($dbh, $del_loc, $del_dev);

  #===== Print device info
  print "<strong>Deleted Device:</strong><br>\n";
  print "&nbsp;&nbsp;Location: $del_loc<br>\n";
  print "&nbsp;&nbsp;Device Name: $del_dev<br>\n";
  print "&nbsp;&nbsp;Type: $type<br>\n";
  print "&nbsp;&nbsp;Hostname: $hostname<br>\n";
  print "&nbsp;&nbsp;Port: $port<br>\n";
  print "&nbsp;&nbsp;Description: $descr<br>\n";
  print "<p>\n";

  if ($errors)
  {
    print "ERROR: $errors!<p>\n";
  }
  else
  {
    print "Device deleted successfully!<p>\n";
  }

  #===== Return to start link
  print qq~<a href="$this_script">Return to Start</a>\n~;

  #===== Write to the log
  my $log_str = "Deleted device $del_loc:$del_dev in database.\n";
  writeToLog($log_str);

  return;

} # end of processDeleteDevice()


#===========================================================
#    W R I T E  T O  L O G
#===========================================================
sub writeToLog(;$)
{
  my $output_str = shift;

  if ($output_str)
  {
    print $LOG getTimestamp() . ": ";
    print $LOG $output_str;
  }
  
  return;
  
} # end of writeToLog()


#===========================================================
#    G E T  T I M E S T A M P
#===========================================================
sub getTimestamp()
{
  return(strftime("%Y-%m-%d %H:%M:%S", localtime));
} # end of getTimestamp()


#===========================================================
#    P R I N T  J A V A S C R I P T
#===========================================================

sub printJavascript()
{

  print <<"EOF";

<script language="Javascript">


//========================================================
//    H A N D L E  M O D E
//========================================================
function handleMode()
{ 
  if (document.myForm.mode.value == 'edit_device' ||
      document.myForm.mode.value == 'delete_device')
  {
    document.myForm.selected_dev.disabled = false;
  }
  else
  {
    document.myForm.selected_dev.disabled = true;
  }

  return;

} // end of handleMode()


//========================================================
//    H A N D L E  L O C A T I O N
//========================================================
function handleLocation()
{
  if (document.myForm.device_location.value == "")
  {
    var new_loc = prompt("Enter the new device location:", "");
    var new_opt = document.createElement('option');
    new_opt.text = new_loc;
    new_opt.value = new_loc;
    var n_opts = document.myForm.device_location.length;
    var old_opt = document.myForm.device_location.options[n_opts-1];
    try {
      document.myForm.device_location.add(new_opt, old_opt); // doesn't work in IE
    }
    catch(ex) {
      document.myForm.device_location.add(new_opt, n_opts-1); // IE only
    }
    document.myForm.device_location.options[n_opts-1].selected = true;
  } // if we are adding a new location

  return;

} // end of handleLocation()


function submitModeSelect()
{
  if ((document.myForm.mode.value == 'edit_device' ||
       document.myForm.mode.value == 'delete_device') &&
      document.myForm.selected_dev.value == "")
  {
    alert("Must select a device to edit or delete!");
    document.myForm.selected_dev.focus();
    return;
  } // if we're editing or deleting a device

  document.myForm.submit();

  return;

} // end of submitModeSelect()


function submitEditDevice()
{
  var rExp = /^[0-9]+\$/; // all integers

  if (document.myForm.device_name.value == '')
  {
    alert("Must supply device name!");
    document.myForm.device_name.focus();
    return;
  }
  if (document.myForm.hostname.value == '')
  {
    alert("Must supply hostname (e.g. 'myhost.college.edu')!");
    document.myForm.hostname.focus();
    return;
  }
  if (document.myForm.device_port.value == '')
  {
    alert("Must supply a port number!");
    document.myForm.device_port.focus();
    return;
  }
  if (!rExp.test(document.myForm.device_port.value))
  {
    alert("Port number must be an integer!");
    document.myForm.device_port.focus();
    return;
  }

  if (document.myForm.descr.value == '')
  {
    alert("Please supply a short description of this device.");
    document.myForm.descr.focus();
    return;
  }

  document.myForm.submit();

  return;

} // end of submitEditDevice()


function submitDeleteDevice()
{
  if (confirm("Sure you want to delete this device?"))
  {
    document.myForm.submit();
  }
  return;

} // end of submitDeleteDevice()

EOF

  print "</script>\n";

} # end of printJavascript()
