#!/usr/bin/perl -w

#=============================================================================
#
#    gims_ajax.cgi
#
#    Written by Charles Thomas (cthomas@wisc.edu)
#
#    $Id: gims_ajax.cgi,v 1.6 2010/03/02 21:43:12 cthomas Exp $
#
#=============================================================================


#=============================================================================
#    U S E  /  R E Q U I R E
#=============================================================================
use CGI qw(:standard);
use CGI::Carp ('fatalsToBrowser');         # Send any errors to the browser
use lib '/usr/testbed/www/gims';
use GIMSDBUtils;
use strict;


#=============================================================================
#    P R O T O T Y P E S
#=============================================================================
sub getParams();
sub createNewExperiment($$$$$);
sub getExpSettings($$;$;$);
sub verifyInput($);


#=============================================================================
#    P A R A M E T E R S
#=============================================================================
my $function = '';
my $exp_id = '';
my $location = '';
my $device_name = '';
my $user = '';


#=============================================================================
#    G L O B A L  V A R I A B L E S
#=============================================================================
my $dbh = undef;            # database handle
my @output = ();
my $csv_safe_str = '';


#=============================================================================
#    M A I N
#=============================================================================

#===== Get database handle
$dbh = GIMSDBUtils::getDBHandle(); 
die "Connection to database failed!\n" unless ref($dbh);

getParams();

print "Content-type: text/plain\n\n";

#===== genExpID
if ($function eq 'genExpID')
{  
  my $exp_id = GIMSDBUtils::getNewExpID($dbh, 'EXP');  
  print "$function\n";
  print "$exp_id\n";
} # function is genExID

#===== createNewExperiment
elsif ($function eq 'createNewExperiment')
{
  my $result = createNewExperiment($dbh, 
                                   $exp_id, 
                                   $location,
                                   $device_name,
                                   $user);

  print "$function\n";
  print "$result\n";
} # function is createNewExperiment

#===== getExpIDs
elsif ($function eq 'getExpIDs')
{
  my %exp_info = ();
  my $result = '';

  GIMSDBUtils::getExperimentInfo($dbh, \%exp_info, '', 'true');

  #===== Store each piece of information
  foreach my $exp_id (sort keys %exp_info)
  {
    my $status = $exp_info{$exp_id}{'status'};
    my $last_update = $exp_info{$exp_id}{'last_update'};
    $last_update =~ s/ /_/g;
    $result .= ' ' if $result;
    $result .= "${exp_id}|${status}|${last_update}";
  }

  print "$function\n";
  print "$result\n";
  
} # getExpIDs

#===== getExpDevices
elsif ($function eq 'getExpDevices')
{
  my %devices = ();
  my $result = '';

  GIMSDBUtils::getExpDevices($dbh, \%devices, $exp_id);
  
  #===== Package up our results
  foreach my $loc (keys %{$devices{$exp_id}})
  {
    foreach my $dev (keys %{$devices{$exp_id}{$loc}})
    {
      $result .= ' ' if $result;
      $result .= "${loc}:${dev}";
    } # for each device
  } # for each location

  print "$function\n";
  print "$result\n";
} # getExpDevices

#===== getAllDevices
elsif ($function eq 'getAllDevices')
{
  my %devices = ();
  my $reply = '';

  #===== Will return only devices for a location if location was given
  #      Will return all devices otherwise
  GIMSDBUtils::getDeviceInfo($dbh, \%devices, $location);

  foreach my $loc (sort keys %devices)
  {
    foreach my $dev (sort keys %{$devices{$loc}})
    {
      $reply .= " " if $reply;
      $reply .= "${loc}:${dev}";
    } # for each device
  } # for each location

  print "$function\n";
  print "$reply\n";
} # getAllDevices

#===== getExpSettings
elsif ($function eq 'getExpSettings')
{
  my $result = getExpSettings($dbh, $exp_id, $location, $device_name); 
  print "$function\n";
  print "$result\n";
} # getExpSettings

#===== Unknown function
else
{
  print "$function\n";
  print "ERROR: Unknown command $function.\n";
} # unknown function

exit;



#################### START SUBROUTINES ##########################


#=======================================================================
#    G E T  P A R A M S
# 
#    Get the parameters the user has entered.
#=======================================================================
sub getParams()
{
    
  if (param('function')) { $function = param('function'); } 
  if (param('exp_id')) { $exp_id  = param('exp_id'); } 
  if (param('location')) { $location = param('location'); } 
  if (param('device_name')) { $device_name = param('device_name'); } 
  if (param('user')) { $user = param('user'); } 
 
  #===== Verify Input
  verifyInput($function) if ($function);
  verifyInput($exp_id) if ($exp_id);
  verifyInput($location) if ($location);
  verifyInput($device_name) if ($device_name);
  verifyInput($user) if ($user);
  
  $user = 'unknown' unless $user; 

  return;

} # end of getParams()


#=======================================================================
#    C R E A T E  N E W  E X P E R I M E N T
# 
#    Arg1: Database handle
#    Arg2: The ExperimentID
#    Arg3: A location
#    Arg4: A device name
#    Arg5: The user creating the experiment.
#    Returns: The function name on the first line and a
#             status on the second line
#=======================================================================
sub createNewExperiment($$$$$)
{
  my $dbh = shift or 
    die "Must supply database handle to createNewExperiment()!\n";
  my $exp_id = shift or
    die "Must supply ExperimentID to createNewExeriment()!\n";
  my $location = shift or
    die "Must supply location to createNewExperiment()!\n";
  my $device_name = shift or
    die "Must supply device name to createNewExperiment()!\n";
  my $user = shift or
    die "Must supply user to createNewExeriment()!\n";
  my $result = '';

  #===== Check to see if this ExperimentID exists
  my $exists = GIMSDBUtils::expIDExists($dbh, $exp_id);

  if (!$exists)
  {
    #===== Add experiment first
    my $errors = GIMSDBUtils::addExperiment($dbh, $exp_id, $user);

    if ($errors)
    {
      $errors =~ s/\n/ /g; # remove newlines
      $result .= "ERROR adding experiment: $errors! ";
    }
    else
    {
      $result .= "Experiment $exp_id created.  ";
    }

    #===== Add experiment device
    $errors = GIMSDBUtils::addExpDevice($dbh, $exp_id, $location, $device_name); 
    if ($errors)
    {
      $errors =~ s/\n/ /g; # remove newlines
      $result .= "ERROR adding device: $errors!";
    }
    else
    {
      $result .= "Added device ${location}:${device_name} ";
      $result .= "to the experiment.";
    }
  } # if the ExpID doesn't exist
  else
  {
    $result .= "ERROR: ExpID $exp_id found in database!\n";
  } # if the ExpID already is in the DB

  return($result);

} # end of createNewExperiment


#=======================================================================
#   G E T  E X P  S E T T I N G S
#
#   Arg1: A database handle
#   Arg2: The ExperimentID
#   Arg3: A location (optional)
#   Arg4: A device name (optional)
#   Returns: Function name on first line
#            CSV-safe pair on each line thereafter which contains
#            location:device pair in element 0
#            XML/RPC object with settings in element 1 
#=======================================================================
sub getExpSettings($$;$;$)
{
  my $dbh = shift or 
    die "Must supply database handle to getExpSettings()!\n";
  my $exp_id = shift or
    die "Must supply ExperimentID to getExpSettings()!\n";
  my $location = shift;
  my $device_name = shift;
  my %settings = ();
  my $result = '';

  GIMSDBUtils::getExpDeviceInfo($dbh, 
                                \%settings, 
                                $exp_id, 
                                $location, 
                                $device_name);

  #===== Store each result in a CSV-safe way
  foreach my $exp (sort keys %settings)
  {
    foreach my $loc (sort keys %{$settings{$exp}})
    {
      foreach my $dev (sort keys %{$settings{$exp}{$loc}})
      {
        my $loc_dev = "${loc}:${dev}";
        my $setting = $settings{$exp}{$loc}{$dev}{'settings'};
        $result .= $setting;
      } # for each device  
    } # for each location
  } # for each ExpID

  return($result);

} # end of getExpSettings()


#=======================================================================
#   V E R I F Y  I N P U T
#
#   Arg1: The input to be verified
#=======================================================================
sub verifyInput($)
{
  my $input = shift or
    die "Must supply input to verifyInput()!\n";
    
  if ($input =~ /\;/ or
      $input =~ /\%/ or
      $input =~ /\^/ or
      $input =~ /\&/ or
      $input =~ /\|/)
  {
    die("Illegal character in user input $input!\n");
  }
  
  return;
  
} # end of verifyInput()


