#! /usr/bin/perl -w

#===== Use/Require
use Socket;
use XML::Simple;
use Data::Dumper;
use XmlRpcUtils;
use strict;

#===== Prototypes
sub getResponse($);
sub getExperimentStatus($);

my $port = 1101;

#===== Get the protocol
my $proto = getprotobyname('tcp');

#===== Create the socket
socket(SERVER, PF_INET, SOCK_STREAM, $proto) or die "socket: $!";

#===== Set socket options
setsockopt(SERVER, SOL_SOCKET, SO_REUSEADDR, 1) or die "setsock: $!";

#===== Bind to local port
my $paddr = sockaddr_in($port, INADDR_ANY);
bind(SERVER, $paddr) or die "bind: $!";

#===== Listen for incoming connections
listen(SERVER, SOMAXCONN) or die "listen: $!";
print "SERVER started on port $port \n";
my $client_addr;
while ($client_addr = accept(CLIENT, SERVER)) 
{
  my $msg = '';

  #===== Set the CLIENT filehandle to unbuffered mode
  my $old_fh = select(CLIENT);
  $| = 1;
  select($old_fh);
  print "Connecting to client...\n";

  #===== Get the message from the client
  while(<CLIENT>)
  {
    #===== Get rid of HTTP crap
    next if m/^POST/;
    next if m/^Content-Type/;
    next if m/^Content-length/;
    
    #===== Add this line to message
    $msg .= $_;

    #===== Exit if we've read the end of the method call
    last if m/\<\/methodCall\>/; # exit on last line
  }
  
  print "From Client:\n";
  print $msg;

  #===== Formulate a response
  my $response = getResponse($msg);
  
  #===== Write response out socket to client
  print "Sending reply...\n";
  print CLIENT $response;
  print "$response\n";

  #===== Kill the connection and terminate client
  print "Killing the socket...\n";
  close CLIENT; 
} # while we have incoming messages

exit;

############# BEGIN SUBROUTINES #################

sub getResponse($)
{
  my $xml_code = shift or
    die "Must supply XML code to getResponse()!\n";
  my $response = '';

  #===== Strip off header info
  my @xml_lines = split(/\n/, $xml_code);
  @xml_lines = grep(!/^POST/, @xml_lines);
  @xml_lines = grep(!/^Content-/, @xml_lines);
  @xml_lines = grep(!/^\n*$/, @xml_lines);
  $xml_code = join('', @xml_lines);

  my $xml_ref = XMLin($xml_code);
  
  # Debug
  #$Data::Dumper::Indent = 1;
  #print Dumper($xml_ref);
    
  my $response_str = '';
  my $error = 0;
    
  if ($xml_ref->{methodName} eq 'GetLocationInfo')
  {
    $response_str = "standin doesn't process GetLocationInfo requests!";
  }
  elsif ($xml_ref->{methodName} eq 'StartExperiment')
  {
    $response_str = 'Starting capture!';
  }
  elsif ($xml_ref->{methodName} eq 'PauseExperiment')
  {
    $response_str = 'Pausing capture!';
  }
  elsif ($xml_ref->{methodName} eq 'StopExperiment')
  {
    $response_str = 'Stopping capture!';
  }
  elsif ($xml_ref->{methodName} eq 'ConfigureExperiment')
  {
    $response_str = 'Configuring Experiment!';
  }
  elsif ($xml_ref->{methodName} eq 'GetExperimentStatus')
  {
    my $exp_id = 'Foo'; # FIX ME
    $response_str = getExperimentStatus($exp_id);
  }
  else
  {
    $response_str = "Don't understand command $xml_ref->{methodName}!";
    $error = 1;
  }
 
  #===== Finish encapsulation 
  if ($error)
  {
    $response = XmlRpcUtils::writeFaultXML(-1, $response_str);
  }
  else
  {
    $response = XmlRpcUtils::writeMethodResponseXML($response_str);
  }
  
  return($response);
  
} # end of getResponse()


sub getExperimentStatus($)
{
  my $experiment_id = shift or
    die "Must supply experimentID to getExperimentStatus()!\n";
  my $status = '';

  $status = "Getting experiment status...\n\n";

  #==== FIX ME
  # Return %CPU utilization
  # Return num pkts captured
  # Return bytes sent to storage
  # Return Time elapsed since beginning of experiment
  # Etc.

  return($status);

} # end of getExperimentStatus()
